"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ScriptManagerAPI = exports.ScriptManager = void 0;

var _events = _interopRequireDefault(require("events"));

var _reactNative = require("react-native");

var _Script = require("./Script");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* globals __DEV__, __webpack_require__ */
const CACHE_KEY = `Repack.ScriptManager.Cache.v3.${__DEV__ ? 'debug' : 'release'}`;
/**
 * A manager to ease resolution, downloading and executing additional code from:
 * - arbitrary JavaScript scripts
 * - Webpack chunks
 * - Webpack bundles
 * - Webpack MF containers
 *
 * An instance of `ScriptManagerAPI` class is exported under `ScriptManager` constant.
 *
 * __You should always use `ScriptManager` constant instead of constructing `new ScriptManagerAPI(...)` yourself!__
 *
 * This API is mainly useful, if you are working with any form of Code Splitting.
 *
 * `ScriptManager` is also an `EventEmitter` and emits the following events:
 * - `resolving` with `{ scriptId, caller }`
 * - `resolved` with `scriptLocator: NormalizedScriptLocator`
 * - `prefetching` with `scriptLocator: NormalizedScriptLocator`
 * - `loading` with `scriptLocator: NormalizedScriptLocator`
 * - `loaded` with `scriptLocator: NormalizedScriptLocator`
 * - `error` with `error: Error`
 *
 * Example of using this API with async Webpack chunk:
 * ```js
 * import * as React from 'react';
 * import { ScriptManager, Script } from '@callstack/repack/client';
 *
 * ScriptManager.configure({
 *   resolve: async (scriptId) => {
 *     if (__DEV__) {
 *       return {
 *         url: Script.getDevServerURL(scriptId);
 *         cache: false,
 *       };
 *     }
 *
 *     return {
 *       url: Script.getRemoteURL(`http://domain.exaple/apps/${scriptId}`),
 *     };
 *   },
 * });
 *
 * // ScriptManager.loadScript is called internally when running `import()`
 * const TeacherModule = React.lazy(() => import('./Teacher.js'));
 * const StudentModule = React.lazy(() => import('./Student.js'));
 *
 * export function App({ role }) {
 *   if (role === 'teacher') {
 *     return <TeacherModule />;
 *   }
 *
 *   return <StudentModule />
 * }
 * ```
 */

class ScriptManagerAPI extends _events.default {
  /**
   * Constructs instance of `ScriptManagerAPI`.
   * Should not be used directly.
   *
   * @internal
   */
  constructor(nativeModule) {
    super();
    this.nativeModule = nativeModule;
  }
  /**
   * Configures `ScriptManager` to be able to resolve location of scripts.
   * Optionally, it also allows to set up caching to avoid over-fetching.
   *
   * @param config Configuration options.
   */


  configure(config) {
    this.storage = config.storage;
    this.resolve = config.resolve;

    __webpack_require__.repack.loadScriptCallback.push = ((parentPush, ...data) => {
      this.emit('_loaded', data[0]);
      return parentPush(...data);
    }).bind(null, __webpack_require__.repack.loadScriptCallback.push.bind(__webpack_require__.repack.loadScriptCallback));

    __webpack_require__.repack.scriptManager = this;
  }

  async initCache() {
    if (!this.cache) {
      var _this$storage;

      const cache = JSON.parse((await ((_this$storage = this.storage) === null || _this$storage === void 0 ? void 0 : _this$storage.getItem(CACHE_KEY))) ?? '{}');
      this.cache = cache ?? undefined;
    }
  }

  async saveCache() {
    var _this$storage2;

    await ((_this$storage2 = this.storage) === null || _this$storage2 === void 0 ? void 0 : _this$storage2.setItem(CACHE_KEY, JSON.stringify(this.cache)));
  }

  handleError(error, message, ...args) {
    console.error(message, error.message, ...args);
    this.emit('error', {
      message,
      args,
      originalError: error
    });
    throw error;
  }
  /**
   * Resolves a {@link Script} instance with normalized locator data.
   *
   * Use `ScriptManager.on('resolving', ({ scriptId, caller }) => { })` to listen for when
   * the script resolution begins.
   *
   * Use `ScriptManager.on('resolved', (scriptLocator) => { })` to listen for when
   * the script's locator data is resolved.
   *
   * @param scriptId Id of the script to resolve.
   * @param caller Name of the calling script - it can be for example: name of the bundle, chunk or container.
   */


  async resolveScript(scriptId, caller) {
    await this.initCache();

    try {
      var _this$cache;

      if (!this.resolve) {
        throw new Error('No script resolver was provided. Did you forget to add `ScriptManager.configure({ resolve: ... })`?');
      }

      this.emit('resolving', {
        scriptId,
        caller
      });
      const locator = await this.resolve(scriptId, caller);

      const script = _Script.Script.from(locator, false);

      if (!((_this$cache = this.cache) !== null && _this$cache !== void 0 && _this$cache[scriptId])) {
        script.locator.fetch = true;
        this.cache = this.cache ?? {};
        this.cache[scriptId] = script.getCacheData();
        await this.saveCache();
      } else if (script.shouldRefetch(this.cache[scriptId])) {
        script.locator.fetch = true;
        this.cache[scriptId] = script.getCacheData();
        await this.saveCache();
      }

      this.emit('resolved', script.locator);
      return script;
    } catch (error) {
      this.handleError(error, '[ScriptManager] Failed while resolving script locator:');
    }
  }
  /**
   * Resolves given script's location, downloads and executes it.
   * The execution of the code is handled internally by threading in React Native.
   *
   * Use `ScriptManager.on('loading', (scriptLocator) => { })` to listen for when
   * the script is about to be loaded.
   *
   * Use `ScriptManager.on('loaded', (scriptLocator) => { })` to listen for when
   * the script is loaded.
   *
   * @param scriptId Id of the script to load.
   * @param caller Name of the calling script - it can be for example: name of the bundle, chunk or container.
   */


  async loadScript(scriptId, caller) {
    let script = await this.resolveScript(scriptId, caller);
    return await new Promise((resolve, reject) => {
      (async () => {
        const onLoaded = data => {
          if (data === scriptId) {
            this.emit('loaded', script.locator);
            resolve();
          }
        };

        try {
          this.emit('loading', script.locator);
          this.on('_loaded', onLoaded);
          await this.nativeModule.loadScript(scriptId, script.locator);
        } catch (error) {
          const {
            code
          } = error;
          this.handleError(error, '[ScriptManager] Failed to load script:', code ? `[${code}]` : '', script.locator);
        } finally {
          this.removeListener('_loaded', onLoaded);
        }
      })().catch(error => {
        reject(error);
      });
    });
  }
  /**
   * Resolves given script's location and downloads it without executing.
   * This function can be awaited to detect if the script was downloaded and for error handling.
   *
   * Use `ScriptManager.on('prefetching', (scriptLocator) => { })` to listen for when
   * the script's prefetch beings.
   *
   * @param scriptId Id of the script to prefetch.
   * @param caller Name of the calling script - it can be for example: name of the bundle, chunk or container.
   */


  async prefetchScript(scriptId, caller) {
    let script = await this.resolveScript(scriptId, caller);

    try {
      this.emit('prefetching', script.locator);
      await this.nativeModule.prefetchScript(scriptId, script.locator);
    } catch (error) {
      const {
        code
      } = error;
      this.handleError(error, '[ScriptManager] Failed to prefetch script:', code ? `[${code}]` : '', script.locator);
    }
  }
  /**
   * Clears the cache (if configured in {@link ScriptManager.configure}) and removes downloaded
   * files for given scripts from the filesystem. This function can be awaited to detect if the
   * scripts were invalidated and for error handling.
   *
   * Use `ScriptManager.on('invalidated', (scriptIds) => { })` to listen for when
   * the invalidation completes.
   *
   * @param scriptIds Array of script ids to clear from cache and remove from filesystem.
   */


  async invalidateScripts(scriptIds = []) {
    try {
      await this.initCache();
      const ids = scriptIds ?? Object.keys(this.cache ?? {});

      for (const scriptId of ids) {
        var _this$cache2;

        (_this$cache2 = this.cache) === null || _this$cache2 === void 0 ? true : delete _this$cache2[scriptId];
      }

      await this.saveCache();
      await this.nativeModule.invalidateScripts(ids);
      this.emit('invalidated', ids);
    } catch (error) {
      const {
        code
      } = error;
      this.handleError(error, '[ScriptManager] Failed to invalidate scripts:', code ? `[${code}]` : '');
    }
  }

}

exports.ScriptManagerAPI = ScriptManagerAPI;
const ScriptManager = new ScriptManagerAPI(_reactNative.NativeModules.ScriptManager);
exports.ScriptManager = ScriptManager;
//# sourceMappingURL=ScriptManager.js.map