"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.OutputPlugin = void 0;

var _path = _interopRequireDefault(require("path"));

var _webpack = _interopRequireDefault(require("webpack"));

var _env = require("../../env");

var _AssetsCopyProcessor = require("./utils/AssetsCopyProcessor");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Plugin for copying generated files (bundle, chunks, assets) from Webpack's built location to the
 * React Native application directory, so that the files can be packed together into the `ipa`/`apk`.
 *
 * @category Webpack Plugin
 */
class OutputPlugin {
  /**
   * Constructs new `OutputPlugin`.
   *
   * @param config Plugin configuration options.
   */
  constructor(config) {
    this.config = config;

    if (!this.config.platform) {
      throw new Error('Missing `platform` option in `OutputPlugin`');
    }
  }
  /**
   * Apply the plugin.
   *
   * @param compiler Webpack compiler instance.
   */


  apply(compiler) {
    const cliOptions = JSON.parse(process.env[_env.CLI_OPTIONS_ENV_KEY] ?? 'null'); // Noop when running from Webpack CLI or when running with dev server

    if (!cliOptions || 'start' in cliOptions.arguments || this.config.devServerEnabled) {
      return;
    }

    const logger = compiler.getInfrastructureLogger('ReactNativeOutputPlugin');
    const args = cliOptions.arguments.bundle;
    let {
      bundleOutput,
      assetsDest = '',
      sourcemapOutput = ''
    } = args;

    if (!_path.default.isAbsolute(bundleOutput)) {
      bundleOutput = _path.default.join(cliOptions.config.root, bundleOutput);
    }

    const bundleOutputDir = _path.default.dirname(bundleOutput);

    if (!sourcemapOutput) {
      sourcemapOutput = `${bundleOutput}.map`;
    }

    if (!_path.default.isAbsolute(sourcemapOutput)) {
      sourcemapOutput = _path.default.join(cliOptions.config.root, sourcemapOutput);
    }

    if (!assetsDest) {
      assetsDest = bundleOutputDir;
    }

    let remoteChunksOutput = this.config.remoteChunksOutput;

    if (remoteChunksOutput && !_path.default.isAbsolute(remoteChunksOutput)) {
      remoteChunksOutput = _path.default.join(cliOptions.config.root, remoteChunksOutput);
    }

    logger.debug('Detected output paths:', {
      bundleOutput,
      sourcemapOutput,
      assetsDest,
      remoteChunksOutput
    });

    const isLocalChunk = chunkId => _webpack.default.ModuleFilenameHelpers.matchObject({
      include: this.config.localChunks ?? []
    }, chunkId);

    let entryGroup;
    let entryChunk;
    const entryChunkName = this.config.entry ?? 'main';
    const localChunks = [];
    const remoteChunks = [];
    compiler.hooks.compilation.tap('OutputPlugin', compilation => {
      compilation.hooks.processAssets.tap({
        name: 'OutputPlugin',
        stage: _webpack.default.Compilation.PROCESS_ASSETS_STAGE_ADDITIONS
      }, () => {
        var _entryGroup;

        entryGroup = compilation.chunkGroups.find(group => group.isInitial());
        entryChunk = (_entryGroup = entryGroup) === null || _entryGroup === void 0 ? void 0 : _entryGroup.chunks.find(chunk => chunk.name === entryChunkName);
        const sharedChunks = new Set();

        for (const chunk of compilation.chunks) {
          var _chunk$id;

          // Do not process shared chunks right now.
          if (sharedChunks.has(chunk)) {
            continue;
          }

          [...chunk.getAllInitialChunks()].filter(sharedChunk => sharedChunk !== chunk).forEach(sharedChunk => {
            sharedChunks.add(sharedChunk);
          }); // Entry chunk

          if (entryChunk && entryChunk === chunk) {
            localChunks.push(chunk);
          } else if (isLocalChunk(chunk.name ?? ((_chunk$id = chunk.id) === null || _chunk$id === void 0 ? void 0 : _chunk$id.toString()))) {
            localChunks.push(chunk);
          } else {
            remoteChunks.push(chunk);
          }
        } // Process shared chunks to add them either as local or remote chunk.


        for (const sharedChunk of sharedChunks) {
          var _sharedChunk$id;

          const isUsedByLocalChunk = localChunks.some(localChunk => {
            return [...localChunk.getAllInitialChunks()].includes(sharedChunk);
          });

          if (isUsedByLocalChunk || isLocalChunk(sharedChunk.name ?? ((_sharedChunk$id = sharedChunk.id) === null || _sharedChunk$id === void 0 ? void 0 : _sharedChunk$id.toString()))) {
            localChunks.push(sharedChunk);
          } else {
            remoteChunks.push(sharedChunk);
          }
        }

        if (!entryChunk) {
          throw new Error('Cannot infer entry chunk - this should have not happened.');
        }

        const mainBundleAssetName = [...entryChunk.files][0];
        compilation.updateAsset(mainBundleAssetName, source => new _webpack.default.sources.ConcatSource(`var __CHUNKS__=${JSON.stringify({
          local: localChunks.map(localChunk => {
            var _localChunk$id;

            return localChunk.name ?? ((_localChunk$id = localChunk.id) === null || _localChunk$id === void 0 ? void 0 : _localChunk$id.toString());
          })
        })};`, '\n', source));
      });
    });
    compiler.hooks.afterEmit.tapPromise('OutputPlugin', async compilation => {
      const outputPath = compilation.outputOptions.path;

      if (!outputPath) {
        throw new Error('Cannot infer output path from compilation');
      }

      const localAssetsCopyProcessor = new _AssetsCopyProcessor.AssetsCopyProcessor({
        platform: this.config.platform,
        compilation,
        outputPath,
        bundleOutput,
        bundleOutputDir,
        sourcemapOutput,
        assetsDest,
        logger
      });
      const remoteAssetsCopyProcessor = new _AssetsCopyProcessor.AssetsCopyProcessor({
        platform: this.config.platform,
        compilation,
        outputPath,
        bundleOutput: '',
        bundleOutputDir: remoteChunksOutput ?? '',
        sourcemapOutput: '',
        assetsDest: remoteChunksOutput ?? '',
        logger
      });

      for (const chunk of localChunks) {
        // Process entry chunk
        localAssetsCopyProcessor.enqueueChunk(chunk, {
          isEntry: entryChunk === chunk
        });
      }

      if (remoteChunksOutput) {
        for (const chunk of remoteChunks) {
          remoteAssetsCopyProcessor.enqueueChunk(chunk, {
            isEntry: false
          });
        }
      }

      await Promise.all([...localAssetsCopyProcessor.execute(), ...remoteAssetsCopyProcessor.execute()]);
    });
  }

}

exports.OutputPlugin = OutputPlugin;
//# sourceMappingURL=OutputPlugin.js.map