import webpack from 'webpack';
import type { DevServerOptions, WebpackPlugin } from '../../types';
import { LoggerPluginConfig } from './LoggerPlugin';
import { OutputPluginConfig } from './OutputPlugin';
/**
 * {@link RepackPlugin} configuration options.
 */
export interface RepackPluginConfig {
    /** Compilation mode. */
    mode: 'development' | 'production';
    /** Target application platform. */
    platform: string;
    /**
     * Development server configuration options.
     * Used to configure `@callstack/repack-dev-server`.
     *
     * If `undefined`, then development server will not be used.
     */
    devServer?: DevServerOptions;
    /**
     * Whether source maps should be generated. Defaults to `true`.
     *
     * Setting this to `false`, disables any source map generation.
     */
    sourceMaps?: boolean;
    /**
     * Options to configure {@link OutputPlugin}'s `localChunks`, `remoteChunksOutput` and `entry`.
     */
    output?: Pick<OutputPluginConfig, 'localChunks' | 'remoteChunksOutput' | 'entry'>;
    /**
     * Options to configure {@link LoggerPlugin}'s `output`.
     *
     * Setting this to `false` disables {@link LoggerPlugin}.
     */
    logger?: LoggerPluginConfig['output'] | boolean;
}
/**
 * Webpack plugin, which abstracts configuration of other Re.Pack's plugin
 * to make Webpack config more readable.
 *
 * @example Usage in Webpack config (ESM):
 * ```ts
 * import * as Repack from '@callstack/repack';
 *
 * export default (env) => {
 *   const {
 *     mode = 'development',
 *     platform,
 *     devServer = undefined,
 *   } = env;
 *
 *   return {
 *     plugins: [
 *       new Repack.RepackPlugin({
 *         mode,
 *         platform,
 *         devServer,
 *       }),
 *     ],
 *   };
 * };
 * ```
 *
 * Internally, `RepackPlugin` configures the following plugins:
 * - `webpack.DefinePlugin` with `__DEV__` global
 * - {@link AssetsResolverPlugin}
 * - {@link OutputPlugin}
 * - {@link DevelopmentPlugin}
 * - {@link RepackTargetPlugin}
 * - `webpack.SourceMapDevToolPlugin`
 * - {@link LoggerPlugin}
 *
 * `RepackPlugin` provides a sensible defaults, but can be customized to some extent.
 * If you need more control, it's recommended to remove `RepackPlugin` and use other plugins
 * directly, eg:
 * ```ts
 * import React from '@callstack/repack';
 *
 * new Repact.plugins.AssetsResolverPlugin();
 * ```
 *
 * @category Webpack Plugin
 */
export declare class RepackPlugin implements WebpackPlugin {
    private config;
    /**
     * Constructs new `RepackPlugin`.
     *
     * @param config Plugin configuration options.
     */
    constructor(config: RepackPluginConfig);
    /**
     * Apply the plugin.
     *
     * @param compiler Webpack compiler instance.
     */
    apply(compiler: webpack.Compiler): void;
}
